@@ -3,6 +3,15 @@
 require_relative 'helper'
 require 'timeout'
 
+separate_testing do
+  require_relative '../lib/rack/multipart'
+  require_relative '../lib/rack/lint'
+  require_relative '../lib/rack/mock_request'
+  require_relative '../lib/rack/query_parser'
+  require_relative '../lib/rack/utils'
+  require_relative '../lib/rack/request'
+end
+
 describe Rack::Multipart do
   def multipart_fixture(name, boundary = "AaB03x")
     file = multipart_file(name)
@@ -26,6 +35,13 @@ describe Rack::Multipart do
     Rack::Multipart.parse_multipart(env).must_be_nil
   end
 
+  it "raises exception if boundary is too long" do
+    env = Rack::MockRequest.env_for("/", multipart_fixture(:content_type_and_no_filename, "A"*71))
+    lambda {
+      Rack::Multipart.parse_multipart(env)
+    }.must_raise Rack::Multipart::Error
+  end
+
   it "parse multipart content when content type present but disposition is not" do
     env = Rack::MockRequest.env_for("/", multipart_fixture(:content_type_and_no_disposition))
     params = Rack::Multipart.parse_multipart(env)
@@ -49,18 +65,49 @@ describe Rack::Multipart do
     params["text"].must_equal "contents"
   end
 
+  it "raises for invalid data preceding the boundary" do
+    env = Rack::MockRequest.env_for '/', multipart_fixture(:preceding_boundary)
+    lambda {
+      Rack::Multipart.parse_multipart(env)
+    }.must_raise Rack::Multipart::EmptyContentError
+  end
+
+  it "ignores initial end boundaries" do
+    env = Rack::MockRequest.env_for '/', multipart_fixture(:end_boundary_first)
+    params = Rack::Multipart.parse_multipart(env)
+    params["files"][:filename].must_equal "foo"
+  end
+
+  it "parse multipart content with different filename and filename*" do
+    env = Rack::MockRequest.env_for '/', multipart_fixture(:filename_multi)
+    params = Rack::Multipart.parse_multipart(env)
+    params["files"][:filename].must_equal "bar"
+  end
+
   it "set US_ASCII encoding based on charset" do
     env = Rack::MockRequest.env_for("/", multipart_fixture(:content_type_and_no_filename))
     params = Rack::Multipart.parse_multipart(env)
     params["text"].encoding.must_equal Encoding::US_ASCII
 
     # I'm not 100% sure if making the param name encoding match the
-    # Content-Type charset is the right thing to do.  We should revisit this.
+    # content-type charset is the right thing to do.  We should revisit this.
     params.keys.each do |key|
       key.encoding.must_equal Encoding::US_ASCII
     end
   end
 
+  it "sets BINARY encoding for invalid charsets" do
+    env = Rack::MockRequest.env_for("/", multipart_fixture(:content_type_and_unknown_charset))
+    params = Rack::Multipart.parse_multipart(env)
+    params["text"].encoding.must_equal Encoding::BINARY
+
+    # I'm not 100% sure if making the param name encoding match the
+    # content-type charset is the right thing to do.  We should revisit this.
+    params.keys.each do |key|
+      key.encoding.must_equal Encoding::BINARY
+    end
+  end
+
   it "set BINARY encoding on things without content type" do
     env = Rack::MockRequest.env_for("/", multipart_fixture(:none))
     params = Rack::Multipart.parse_multipart(env)
@@ -92,17 +139,6 @@ describe Rack::Multipart do
     params['user_sid'].encoding.must_equal Encoding::UTF_8
   end
 
-  it "raise ParamsTooDeepError if the key space is exhausted" do
-    env = Rack::MockRequest.env_for("/", multipart_fixture(:content_type_and_no_filename))
-
-    old, Rack::Utils.key_space_limit = Rack::Utils.key_space_limit, 1
-    begin
-      lambda { Rack::Multipart.parse_multipart(env) }.must_raise(Rack::QueryParser::ParamsTooDeepError)
-    ensure
-      Rack::Utils.key_space_limit = old
-    end
-  end
-
   it "parse multipart form webkit style" do
     env = Rack::MockRequest.env_for '/', multipart_fixture(:webkit)
     env['CONTENT_TYPE'] = "multipart/form-data; boundary=----WebKitFormBoundaryWLHCs9qmcJJoyjKR"
@@ -127,51 +163,13 @@ describe Rack::Multipart do
         wr.write("--AaB03x")
 
         # make the initial boundary a few gigs long
-        longer = "0123456789" * 1024 * 1024
-        (1024 * 1024).times { wr.write(longer) }
-
-        wr.write("\r\n")
-        wr.write('Content-Disposition: form-data; name="a"; filename="a.txt"')
-        wr.write("\r\n")
-        wr.write("Content-Type: text/plain\r\n")
-        wr.write("\r\na")
-        wr.write("--AaB03x--\r\n")
-        wr.close
-      rescue => err # this is EPIPE if Rack shuts us down
-        err
-      end
-    end
-
-    fixture = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
-      "CONTENT_LENGTH" => (1024 * 1024 * 8).to_s,
-      :input => rd,
-    }
-
-    env = Rack::MockRequest.env_for '/', fixture
-    lambda {
-      Rack::Multipart.parse_multipart(env)
-    }.must_raise EOFError
-    rd.close
-
-    err = thr.value
-    err.must_be_instance_of Errno::EPIPE
-    wr.close
-  end
-
-  it "rejects excessive data before boundary" do
-    rd, wr = IO.pipe
-    def rd.rewind; end
-    wr.sync = true
-
-    thr = Thread.new do
-      begin
         longer = "0123456789" * 1024 * 1024
         (1024 * 1024).times do
-           wr.write(longer)
+          while wr.write_nonblock(longer, exception: false) == :wait_writable
+            Thread.pass
+          end
         end
 
-        wr.write("\r\n\r\n--AaB03x")
         wr.write("\r\n")
         wr.write('content-disposition: form-data; name="a"; filename="a.txt"')
         wr.write("\r\n")
@@ -193,7 +191,7 @@ describe Rack::Multipart do
     env = Rack::MockRequest.env_for '/', fixture
     lambda {
       Rack::Multipart.parse_multipart(env)
-    }.must_raise(EOFError).message.must_equal "multipart boundary not found within limit"
+    }.must_raise Rack::Multipart::EmptyContentError
     rd.close
 
     err = thr.value
@@ -201,149 +199,6 @@ describe Rack::Multipart do
     wr.close
   end
 
-  it "rejects excessive mime header size" do
-    rd, wr = IO.pipe
-    def rd.rewind; end
-    wr.sync = true
-
-    thr = Thread.new do
-      begin
-        wr.write("\r\n\r\n--AaB03x")
-        wr.write("\r\n")
-        wr.write('content-disposition: form-data; name="a"; filename="a.txt"')
-        wr.write("\r\n")
-        wr.write("content-type: text/plain\r\n")
-        longer = "0123456789" * 1024 * 1024
-        (1024 * 1024).times do
-          wr.write(longer)
-        end
-        wr.write("\r\na")
-        wr.write("--AaB03x--\r\n")
-        wr.close
-      rescue => err # this is EPIPE if Rack shuts us down
-        err
-      end
-    end
-
-    fixture = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
-      "CONTENT_LENGTH" => (1024 * 1024 * 8).to_s,
-      :input => rd,
-    }
-
-    env = Rack::MockRequest.env_for '/', fixture
-    lambda {
-      Rack::Multipart.parse_multipart(env)
-    }.must_raise(EOFError).message.must_equal "multipart mime part header too large"
-    rd.close
-
-    err = thr.value
-    err.must_be_instance_of Errno::EPIPE
-    wr.close
-  end
-
-  it "rejects excessive buffered mime data size in a single parameter" do
-    rd, wr = IO.pipe
-    def rd.rewind; end
-    wr.sync = true
-
-    thr = Thread.new do
-      wr.write("--AaB03x")
-      wr.write("\r\n")
-      wr.write('content-disposition: form-data; name="a"')
-      wr.write("\r\n")
-      wr.write("content-type: text/plain\r\n")
-      wr.write("\r\n")
-      wr.write("0" * 17 * 1024 * 1024)
-      wr.write("--AaB03x--\r\n")
-      wr.close
-      true
-    end
-
-    fixture = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
-      "CONTENT_LENGTH" => (18 * 1024 * 1024).to_s,
-      :input => rd,
-    }
-
-    env = Rack::MockRequest.env_for '/', fixture
-    lambda {
-      Rack::Multipart.parse_multipart(env)
-    }.must_raise(EOFError).message.must_equal "multipart data over retained size limit"
-    rd.close
-
-    thr.value.must_equal true
-    wr.close
-  end
-
-  it "rejects excessive buffered mime data size when split into multiple parameters" do
-    rd, wr = IO.pipe
-    def rd.rewind; end
-    wr.sync = true
-
-    thr = Thread.new do
-      4.times do |i|
-        wr.write("\r\n--AaB03x")
-        wr.write("\r\n")
-        wr.write("content-disposition: form-data; name=\"a#{i}\"")
-        wr.write("\r\n")
-        wr.write("content-type: text/plain\r\n")
-        wr.write("\r\n")
-        wr.write("0" * 4 * 1024 * 1024)
-      end
-      wr.write("\r\n--AaB03x--\r\n")
-      wr.close
-      true
-    end
-
-    fixture = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
-      "CONTENT_LENGTH" => (17 * 1024 * 1024).to_s,
-      :input => rd,
-    }
-
-    env = Rack::MockRequest.env_for '/', fixture
-    lambda {
-  p    Rack::Multipart.parse_multipart(env).keys
-    }.must_raise(EOFError).message.must_equal "multipart data over retained size limit"
-    rd.close
-
-    thr.value.must_equal true
-    wr.close
-  end
-
-  it "allows large nonbuffered mime parameters" do
-    rd, wr = IO.pipe
-    def rd.rewind; end
-    wr.sync = true
-
-    thr = Thread.new do
-      wr.write("\r\n\r\n--AaB03x")
-      wr.write("\r\n")
-      wr.write('content-disposition: form-data; name="a"; filename="a.txt"')
-      wr.write("\r\n")
-      wr.write("content-type: text/plain\r\n")
-      wr.write("\r\n")
-      wr.write("0" * 16 * 1024 * 1024)
-      wr.write("\r\n--AaB03x--\r\n")
-      wr.close
-      true
-    end
-
-    fixture = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
-      "CONTENT_LENGTH" => (17 * 1024 * 1024).to_s,
-      :input => rd,
-    }
-
-    env = Rack::MockRequest.env_for '/', fixture
-    Rack::Multipart.parse_multipart(env)['a'][:tempfile].read.bytesize.must_equal(16 * 1024 * 1024)
-    rd.close
-
-    thr.value.must_equal true
-    wr.close
-  end
-
   # see https://github.com/rack/rack/pull/1309
   it "parse strange multipart pdf" do
     boundary = '---------------------------932620571087722842402766118'
@@ -351,13 +206,14 @@ describe Rack::Multipart do
     data = StringIO.new
     data.write("--#{boundary}")
     data.write("\r\n")
-    data.write('Content-Disposition: form-data; name="a"; filename="a.pdf"')
+    data.write('content-disposition: form-data; name="a"; filename="a.pdf"')
     data.write("\r\n")
-    data.write("Content-Type:application/pdf\r\n")
+    data.write("content-type:application/pdf\r\n")
     data.write("\r\n")
     data.write("-" * (1024 * 1024))
     data.write("\r\n")
     data.write("--#{boundary}--\r\n")
+    data.rewind
 
     fixture = {
       "CONTENT_TYPE" => "multipart/form-data; boundary=#{boundary}",
@@ -369,7 +225,7 @@ describe Rack::Multipart do
     Timeout::timeout(10) { Rack::Multipart.parse_multipart(env) }
   end
 
-  it 'raises an EOF error on content-length mistmatch' do
+  it 'raises an EOF error on content-length mismatch' do
     env = Rack::MockRequest.env_for("/", multipart_fixture(:empty))
     env['rack.input'] = StringIO.new
     assert_raises(EOFError) do
@@ -384,9 +240,9 @@ describe Rack::Multipart do
     params["submit-name-with-content"].must_equal "Berry"
     params["files"][:type].must_equal "text/plain"
     params["files"][:filename].must_equal "file1.txt"
-    params["files"][:head].must_equal "Content-Disposition: form-data; " +
+    params["files"][:head].must_equal "content-disposition: form-data; " +
       "name=\"files\"; filename=\"file1.txt\"\r\n" +
-      "Content-Type: text/plain\r\n"
+      "content-type: text/plain\r\n"
     params["files"][:name].must_equal "files"
     params["files"][:tempfile].read.must_equal "contents"
   end
@@ -398,7 +254,7 @@ describe Rack::Multipart do
         @params = Hash.new{|h, k| h[k.to_s] if k.is_a?(Symbol)}
       end
     end
-    query_parser = Rack::QueryParser.new c, 65536, 100
+    query_parser = Rack::QueryParser.new c, 100
     env = Rack::MockRequest.env_for("/", multipart_fixture(:text))
     params = Rack::Multipart.parse_multipart(env, query_parser)
     params[:files][:type].must_equal "text/plain"
@@ -415,9 +271,9 @@ describe Rack::Multipart do
     params = Rack::Multipart.parse_multipart(env)
     params["file1.txt"][:type].must_equal "text/plain"
     params["file1.txt"][:filename].must_equal "file1.txt"
-    params["file1.txt"][:head].must_equal "Content-Disposition: form-data; " +
+    params["file1.txt"][:head].must_equal "content-disposition: form-data; " +
       "filename=\"file1.txt\"\r\n" +
-      "Content-Type: text/plain\r\n"
+      "content-type: text/plain\r\n"
     params["file1.txt"][:name].must_equal "file1.txt"
     params["file1.txt"][:tempfile].read.must_equal "contents"
   end
@@ -437,9 +293,9 @@ describe Rack::Multipart do
     params["foo"]["submit-name"].must_equal "Larry"
     params["foo"]["files"][:type].must_equal "text/plain"
     params["foo"]["files"][:filename].must_equal "file1.txt"
-    params["foo"]["files"][:head].must_equal "Content-Disposition: form-data; " +
+    params["foo"]["files"][:head].must_equal "content-disposition: form-data; " +
       "name=\"foo[files]\"; filename=\"file1.txt\"\r\n" +
-      "Content-Type: text/plain\r\n"
+      "content-type: text/plain\r\n"
     params["foo"]["files"][:name].must_equal "foo[files]"
     params["foo"]["files"][:tempfile].read.must_equal "contents"
   end
@@ -451,9 +307,9 @@ describe Rack::Multipart do
 
     params["files"][:type].must_equal "image/png"
     params["files"][:filename].must_equal "rack-logo.png"
-    params["files"][:head].must_equal "Content-Disposition: form-data; " +
+    params["files"][:head].must_equal "content-disposition: form-data; " +
       "name=\"files\"; filename=\"rack-logo.png\"\r\n" +
-      "Content-Type: image/png\r\n"
+      "content-type: image/png\r\n"
     params["files"][:name].must_equal "files"
     params["files"][:tempfile].read.length.must_equal 26473
   end
@@ -464,9 +320,9 @@ describe Rack::Multipart do
     params["submit-name"].must_equal "Larry"
     params["files"][:type].must_equal "text/plain"
     params["files"][:filename].must_equal "file1.txt"
-    params["files"][:head].must_equal "Content-Disposition: form-data; " +
+    params["files"][:head].must_equal "content-disposition: form-data; " +
       "name=\"files\"; filename=\"file1.txt\"\r\n" +
-      "Content-Type: text/plain\r\n"
+      "content-type: text/plain\r\n"
     params["files"][:name].must_equal "files"
     params["files"][:tempfile].read.must_equal ""
   end
@@ -476,9 +332,9 @@ describe Rack::Multipart do
     params = Rack::Multipart.parse_multipart(env)
     params["files"][:type].must_equal "text/plain"
     params["files"][:filename].must_equal "fi;le1.txt"
-    params["files"][:head].must_equal "Content-Disposition: form-data; " +
+    params["files"][:head].must_equal "content-disposition: form-data; " +
       "name=\"files\"; filename=\"fi;le1.txt\"\r\n" +
-      "Content-Type: text/plain\r\n"
+      "content-type: text/plain\r\n"
     params["files"][:name].must_equal "files"
     params["files"][:tempfile].read.must_equal "contents"
   end
@@ -490,9 +346,9 @@ describe Rack::Multipart do
     params["submit-name-with-content"].must_equal "Berry"
     params["files"][:type].must_equal "text/plain"
     params["files"][:filename].must_equal "file1.txt"
-    params["files"][:head].must_equal "Content-Disposition: form-data; " +
+    params["files"][:head].must_equal "content-disposition: form-data; " +
       "name=\"files\"; filename=\"file1.txt\"\r\n" +
-      "Content-Type: text/plain\r\n"
+      "content-type: text/plain\r\n"
     params["files"][:name].must_equal "files"
     params["files"][:tempfile].read.must_equal "contents"
   end
@@ -502,9 +358,9 @@ describe Rack::Multipart do
     params = Rack::Multipart.parse_multipart(env)
     params["files"][:type].must_equal "text/plain"
     params["files"][:filename].must_match(/invalid/)
-    head = "Content-Disposition: form-data; " +
+    head = "content-disposition: form-data; " +
       "name=\"files\"; filename=\"invalid\xC3.txt\"\r\n" +
-      "Content-Type: text/plain\r\n"
+      "content-type: text/plain\r\n"
     head = head.force_encoding(Encoding::ASCII_8BIT)
     params["files"][:head].must_equal head
     params["files"][:name].must_equal "files"
@@ -529,7 +385,7 @@ describe Rack::Multipart do
     params["files"][:filename].must_equal "flowers.exe\u0000.jpg"
   end
 
-  it "is robust separating Content-Disposition fields" do
+  it "is robust separating content-disposition fields" do
     env = Rack::MockRequest.env_for("/", multipart_fixture(:robust_field_separation))
     params = Rack::Multipart.parse_multipart(env)
     params["text"].must_equal "contents"
@@ -556,10 +412,10 @@ describe Rack::Multipart do
     params = Rack::Multipart.parse_multipart(env)
     params["files"][:type].must_equal "text/plain"
     params["files"][:filename].must_equal "file1.txt"
-    params["files"][:head].must_equal "Content-Disposition: form-data; " +
+    params["files"][:head].must_equal "content-disposition: form-data; " +
       "name=\"files\"; " +
       'filename="C:\Documents and Settings\Administrator\Desktop\file1.txt"' +
-      "\r\nContent-Type: text/plain\r\n"
+      "\r\ncontent-type: text/plain\r\n"
     params["files"][:name].must_equal "files"
     params["files"][:tempfile].read.must_equal "contents"
   end
@@ -569,8 +425,8 @@ describe Rack::Multipart do
     params = Rack::Multipart.parse_multipart(env)
     params["files"][:type].must_equal "image/jpeg"
     params["files"][:filename].must_equal "genome.jpeg"
-    params["files"][:head].must_equal "Content-Type: image/jpeg\r\n" +
-      "Content-Disposition: attachment; " +
+    params["files"][:head].must_equal "content-type: image/jpeg\r\n" +
+      "content-disposition: attachment; " +
       "name=\"files\"; " +
       "filename=genome.jpeg; " +
       "modification-date=\"Wed, 12 Feb 1997 16:29:51 -0500\";\r\n" +
@@ -584,10 +440,10 @@ describe Rack::Multipart do
     params = Rack::Multipart.parse_multipart(env)
     params["files"][:type].must_equal "application/octet-stream"
     params["files"][:filename].must_equal "escape \"quotes"
-    params["files"][:head].must_equal "Content-Disposition: form-data; " +
+    params["files"][:head].must_equal "content-disposition: form-data; " +
       "name=\"files\"; " +
       "filename=\"escape \\\"quotes\"\r\n" +
-      "Content-Type: application/octet-stream\r\n"
+      "content-type: application/octet-stream\r\n"
     params["files"][:name].must_equal "files"
     params["files"][:tempfile].read.must_equal "contents"
   end
@@ -597,10 +453,10 @@ describe Rack::Multipart do
     params = Rack::Multipart.parse_multipart(env)
     params["files"][:type].must_equal "application/octet-stream"
     params["files"][:filename].must_equal "foo+bar"
-    params["files"][:head].must_equal "Content-Disposition: form-data; " +
+    params["files"][:head].must_equal "content-disposition: form-data; " +
       "name=\"files\"; " +
       "filename=\"foo+bar\"\r\n" +
-      "Content-Type: application/octet-stream\r\n"
+      "content-type: application/octet-stream\r\n"
     params["files"][:name].must_equal "files"
     params["files"][:tempfile].read.must_equal "contents"
   end
@@ -610,10 +466,10 @@ describe Rack::Multipart do
     params = Rack::Multipart.parse_multipart(env)
     params["files"][:type].must_equal "application/octet-stream"
     params["files"][:filename].must_equal "escape \"quotes"
-    params["files"][:head].must_equal "Content-Disposition: form-data; " +
+    params["files"][:head].must_equal "content-disposition: form-data; " +
       "name=\"files\"; " +
       "filename=\"escape %22quotes\"\r\n" +
-      "Content-Type: application/octet-stream\r\n"
+      "content-type: application/octet-stream\r\n"
     params["files"][:name].must_equal "files"
     params["files"][:tempfile].read.must_equal "contents"
   end
@@ -623,8 +479,8 @@ describe Rack::Multipart do
     params = Rack::Multipart.parse_multipart(env)
     params["files"][:type].must_equal "image/jpeg"
     params["files"][:filename].must_equal "\"human\" genome.jpeg"
-    params["files"][:head].must_equal "Content-Type: image/jpeg\r\n" +
-      "Content-Disposition: attachment; " +
+    params["files"][:head].must_equal "content-type: image/jpeg\r\n" +
+      "content-disposition: attachment; " +
       "name=\"files\"; " +
       "filename=\"\\\"human\\\" genome.jpeg\"; " +
       "modification-date=\"Wed, 12 Feb 1997 16:29:51 -0500\";\r\n" +
@@ -640,8 +496,8 @@ describe Rack::Multipart do
     files[:type].must_equal "image/jpeg"
     files[:filename].must_equal "100% of a photo.jpeg"
     files[:head].must_equal <<-MULTIPART
-Content-Disposition: form-data; name="document[attachment]"; filename="100% of a photo.jpeg"\r
-Content-Type: image/jpeg\r
+content-disposition: form-data; name="document[attachment]"; filename="100% of a photo.jpeg"\r
+content-type: image/jpeg\r
     MULTIPART
 
     files[:name].must_equal "document[attachment]"
@@ -655,8 +511,8 @@ Content-Type: image/jpeg\r
     files[:type].must_equal "image/jpeg"
     files[:filename].must_equal "100%a"
     files[:head].must_equal <<-MULTIPART
-Content-Disposition: form-data; name="document[attachment]"; filename="100%a"\r
-Content-Type: image/jpeg\r
+content-disposition: form-data; name="document[attachment]"; filename="100%a"\r
+content-type: image/jpeg\r
     MULTIPART
 
     files[:name].must_equal "document[attachment]"
@@ -670,22 +526,21 @@ Content-Type: image/jpeg\r
     files[:type].must_equal "image/jpeg"
     files[:filename].must_equal "100%"
     files[:head].must_equal <<-MULTIPART
-Content-Disposition: form-data; name="document[attachment]"; filename="100%"\r
-Content-Type: image/jpeg\r
+content-disposition: form-data; name="document[attachment]"; filename="100%"\r
+content-type: image/jpeg\r
     MULTIPART
 
     files[:name].must_equal "document[attachment]"
     files[:tempfile].read.must_equal "contents"
   end
 
-  it "rewinds input after parsing upload" do
-    options = multipart_fixture(:text)
-    input = options[:input]
-    env = Rack::MockRequest.env_for("/", options)
-    params = Rack::Multipart.parse_multipart(env)
-    params["submit-name"].must_equal "Larry"
-    params["files"][:filename].must_equal "file1.txt"
-    input.read.length.must_equal 307
+  it "raises RuntimeError for invalid file path" do
+    proc{Rack::Multipart::UploadedFile.new('non-existant')}.must_raise RuntimeError
+  end
+
+  it "supports uploading files in binary mode" do
+    Rack::Multipart::UploadedFile.new(multipart_file("file1.txt")).wont_be :binmode?
+    Rack::Multipart::UploadedFile.new(multipart_file("file1.txt"), binary: true).must_be :binmode?
   end
 
   it "builds multipart body" do
@@ -704,6 +559,22 @@ Content-Type: image/jpeg\r
     params["files"][:tempfile].read.must_equal "contents"
   end
 
+  it "builds multipart filename with space" do
+    files = Rack::Multipart::UploadedFile.new(multipart_file("space case.txt"))
+    data  = Rack::Multipart.build_multipart("submit-name" => "Larry", "files" => files)
+
+    options = {
+      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
+      "CONTENT_LENGTH" => data.length.to_s,
+      :input => StringIO.new(data)
+    }
+    env = Rack::MockRequest.env_for("/", options)
+    params = Rack::Multipart.parse_multipart(env)
+    params["submit-name"].must_equal "Larry"
+    params["files"][:filename].must_equal "space case.txt"
+    params["files"][:tempfile].read.must_equal "contents"
+  end
+
   it "builds nested multipart body using array" do
     files = Rack::Multipart::UploadedFile.new(multipart_file("file1.txt"))
     data  = Rack::Multipart.build_multipart("people" => [{ "submit-name" => "Larry", "files" => files }])
@@ -805,7 +676,22 @@ Content-Type: image/jpeg\r
     end
   end
 
-  it "reach a multipart limit" do
+  it "treat a multipart limit of 0 as no limit" do
+    begin
+      previous_limit = Rack::Utils.multipart_part_limit
+      Rack::Utils.multipart_part_limit = 0
+
+      env = Rack::MockRequest.env_for '/', multipart_fixture(:three_files_three_fields)
+      params = Rack::Multipart.parse_multipart(env)
+      params['reply'].must_equal 'yes'
+      params['to'].must_equal 'people'
+      params['from'].must_equal 'others'
+    ensure
+      Rack::Utils.multipart_part_limit = previous_limit
+    end
+  end
+
+  it "reach a multipart file limit" do
     begin
       previous_limit = Rack::Utils.multipart_part_limit
       Rack::Utils.multipart_part_limit = 3
@@ -843,8 +729,8 @@ Content-Type: image/jpeg\r
   it "can parse fields with a content type" do
     data = <<-EOF
 --1yy3laWhgX31qpiHinh67wJXqKalukEUTvqTzmon\r
-Content-Disposition: form-data; name="description"\r
-Content-Type: text/plain"\r
+content-disposition: form-data; name="description"\r
+content-type: text/plain"\r
 \r
 Very very blue\r
 --1yy3laWhgX31qpiHinh67wJXqKalukEUTvqTzmon--\r
@@ -871,8 +757,8 @@ EOF
   it "parse very long unquoted multipart file names" do
     data = <<-EOF
 --AaB03x\r
-Content-Type: text/plain\r
-Content-Disposition: attachment; name=file; filename=#{'long' * 100}\r
+content-type: text/plain\r
+content-disposition: attachment; name=file; filename=#{'long' * 100}\r
 \r
 contents\r
 --AaB03x--\r
@@ -889,33 +775,11 @@ contents\r
     params["file"][:filename].must_equal 'long' * 100
   end
 
-  it "limits very long file name extensions in multipart tempfiles" do
-    data = <<-EOF
---AaB03x\r
-content-type: text/plain\r
-content-disposition: attachment; name=file; filename=foo.#{'a' * 1000}\r
-\r
-contents\r
---AaB03x--\r
-    EOF
-
-    options = {
-      "CONTENT_TYPE" => "multipart/form-data; boundary=AaB03x",
-      "CONTENT_LENGTH" => data.length.to_s,
-      :input => StringIO.new(data)
-    }
-    env = Rack::MockRequest.env_for("/", options)
-    params = Rack::Multipart.parse_multipart(env)
-
-    params["file"][:filename].must_equal "foo.#{'a' * 1000}"
-    File.extname(env["rack.tempfiles"][0]).must_equal ".#{'a' * 128}"
-  end
-
   it "parse unquoted parameter values at end of line" do
     data = <<-EOF
 --AaB03x\r
-Content-Type: text/plain\r
-Content-Disposition: attachment; name=inline\r
+content-type: text/plain\r
+content-disposition: attachment; name=inline\r
 \r
 true\r
 --AaB03x--\r
@@ -934,8 +798,8 @@ true\r
   it "parse quoted chars in name parameter" do
     data = <<-EOF
 --AaB03x\r
-Content-Type: text/plain\r
-Content-Disposition: attachment; name="quoted\\\\chars\\"in\rname"\r
+content-type: text/plain\r
+content-disposition: attachment; name="quoted\\\\chars\\"in\rname"\r
 \r
 true\r
 --AaB03x--\r
@@ -968,9 +832,9 @@ true\r
     params["submit-name-with-content"].must_equal "Berry"
     params["files"][:type].must_equal "text/plain"
     params["files"][:filename].must_equal "file1.txt"
-    params["files"][:head].must_equal "Content-Disposition: form-data; " +
+    params["files"][:head].must_equal "content-disposition: form-data; " +
       "name=\"files\"; filename=\"file1.txt\"\r\n" +
-      "Content-Type: text/plain\r\n"
+      "content-type: text/plain\r\n"
     params["files"][:name].must_equal "files"
     params["files"][:tempfile].read.must_equal "contents"
   end
@@ -980,15 +844,15 @@ true\r
 
     data = <<-EOF.dup
 --AaB03x\r
-Content-Type: text/plain\r
+content-type: text/plain\r
 \r
 some text\r
 --AaB03x\r
 \r
 \r
-some more text (I didn't specify Content-Type)\r
+some more text (I didn't specify content-type)\r
 --AaB03x\r
-Content-Type: image/png\r
+content-type: image/png\r
 \r
 #{rack_logo}\r
 --AaB03x--\r
@@ -1002,7 +866,7 @@ Content-Type: image/png\r
     env = Rack::MockRequest.env_for("/", options)
     params = Rack::Multipart.parse_multipart(env)
 
-    params["text/plain"].must_equal ["some text", "some more text (I didn't specify Content-Type)"]
+    params["text/plain"].must_equal ["some text", "some more text (I didn't specify content-type)"]
     params["image/png"].length.must_equal 1
 
     f = Tempfile.new("rack-logo")
