@@ -3,6 +3,14 @@
 require_relative 'helper'
 require 'timeout'
 
+separate_testing do
+  require_relative '../lib/rack/utils'
+  require_relative '../lib/rack/lint'
+  require_relative '../lib/rack/mock_request'
+  require_relative '../lib/rack/request'
+  require_relative '../lib/rack/content_length'
+end
+
 describe Rack::Utils do
 
   def assert_sets(exp, act)
@@ -114,7 +122,7 @@ describe Rack::Utils do
     ex = { "foo" => nil }
     ex["foo"] = ex
 
-    params = Rack::Utils::KeySpaceConstrainedParams.new(65536)
+    params = Rack::Utils::KeySpaceConstrainedParams.new
     params['foo'] = params
     params.to_params_hash.to_s.must_equal ex.to_s
   end
@@ -123,6 +131,22 @@ describe Rack::Utils do
     Rack::Utils.parse_nested_query(nil).must_equal({})
   end
 
+  deprecated "should warn for deprecated QueryParser.make_default call with key_space_limit" do
+    Rack::QueryParser.make_default(1, 1).must_be_kind_of Rack::QueryParser
+  end
+
+  deprecated "should warn using deprecated QueryParser.new call with key_space_limit" do
+    Rack::QueryParser.new(Rack::QueryParser::Params, 1, 1).must_be_kind_of Rack::QueryParser
+  end
+
+  deprecated "should warn using deprecated Rack::Util.key_space_limit=" do
+    Rack::Utils.key_space_limit = 65536
+  end
+
+  deprecated "should warn using deprecated Rack::Util.key_space_limit" do
+    Rack::Utils.key_space_limit
+  end
+
   it "raise an exception if the params are too deep" do
     len = Rack::Utils.param_depth_limit
 
@@ -133,12 +157,6 @@ describe Rack::Utils do
     Rack::Utils.parse_nested_query("foo#{"[a]" * (len - 1)}=bar")
   end
 
-  # ParamsTooDeepError was introduced in the middle of 2.2 releases
-  # and this test is here to ensure backwards compatibility
-  it "ParamsTooDeepError is inherited from originally used RangeError" do
-    (Rack::QueryParser::ParamsTooDeepError < RangeError).must_equal(true)
-  end
-
   it "parse nested query strings correctly" do
     Rack::Utils.parse_nested_query("foo").
       must_equal "foo" => nil
@@ -191,6 +209,8 @@ describe Rack::Utils do
     Rack::Utils.parse_nested_query("foo[]=bar&baz[]=1&baz[]=2&baz[]=3").
       must_equal "foo" => ["bar"], "baz" => ["1", "2", "3"]
 
+    Rack::Utils.parse_nested_query("x[y][z]").
+      must_equal "x" => { "y" => { "z" => nil } }
     Rack::Utils.parse_nested_query("x[y][z]=1").
       must_equal "x" => { "y" => { "z" => "1" } }
     Rack::Utils.parse_nested_query("x[y][z][]=1").
@@ -237,9 +257,8 @@ describe Rack::Utils do
       must_raise(Rack::Utils::ParameterTypeError).
       message.must_equal "expected Array (got String) for param `y'"
 
-    lambda { Rack::Utils.parse_nested_query("foo%81E=1") }.
-      must_raise(Rack::Utils::InvalidParameterError).
-      message.must_equal "invalid byte sequence in UTF-8"
+    Rack::Utils.parse_nested_query("foo%81E=1").
+      must_equal "foo\x81E"=>"1"
   end
 
   it "only moves to a new array when the full key has been seen" do
@@ -254,6 +273,20 @@ describe Rack::Utils do
       ]
   end
 
+  it "handles unexpected use of [ and ] in parameter keys as normal characters" do
+    Rack::Utils.parse_nested_query("[]=1&[a]=2&b[=3&c]=4").
+      must_equal "[]" => "1", "[a]" => "2", "b[" => "3", "c]" => "4"
+
+    Rack::Utils.parse_nested_query("d[[]=5&e][]=6&f[[]]=7").
+      must_equal "d" => {"[" => "5"}, "e]" => ["6"], "f" => { "[" => { "]" => "7" } }
+
+    Rack::Utils.parse_nested_query("g[h]i=8&j[k]l[m]=9").
+      must_equal "g" => { "h" => { "i" => "8" } }, "j" => { "k" => { "l[m]" =>"9" } }
+
+    Rack::Utils.parse_nested_query("l[[[[[[[[]]]]]]]=10").
+      must_equal "l"=>{"[[[[[[["=>{"]]]]]]"=>"10"}}
+  end
+
   it "allow setting the params hash class to use for parsing query strings" do
     begin
       default_parser = Rack::Utils.default_query_parser
@@ -263,7 +296,7 @@ describe Rack::Utils do
           @params = Hash.new{|h, k| h[k.to_s] if k.is_a?(Symbol)}
         end
       end
-      Rack::Utils.default_query_parser = Rack::QueryParser.new(param_parser_class, 65536, 100)
+      Rack::Utils.default_query_parser = Rack::QueryParser.new(param_parser_class, 100)
       h1 = Rack::Utils.parse_query(",foo=bar;,", ";,")
       h1[:foo].must_equal "bar"
       h2 = Rack::Utils.parse_nested_query("x[y][][z]=1&x[y][][w]=2")
@@ -295,9 +328,9 @@ describe Rack::Utils do
     assert_nested_query("my+weird+field=q1%212%22%27w%245%267%2Fz8%29%3F",
                         "my weird field" => "q1!2\"'w$5&7/z8)?")
 
-    Rack::Utils.build_nested_query("foo" => [nil]).must_equal "foo[]"
-    Rack::Utils.build_nested_query("foo" => [""]).must_equal "foo[]="
-    Rack::Utils.build_nested_query("foo" => ["bar"]).must_equal "foo[]=bar"
+    Rack::Utils.build_nested_query("foo" => [nil]).must_equal "foo%5B%5D"
+    Rack::Utils.build_nested_query("foo" => [""]).must_equal "foo%5B%5D="
+    Rack::Utils.build_nested_query("foo" => ["bar"]).must_equal "foo%5B%5D=bar"
     Rack::Utils.build_nested_query('foo' => []).must_equal ''
     Rack::Utils.build_nested_query('foo' => {}).must_equal ''
     Rack::Utils.build_nested_query('foo' => 'bar', 'baz' => []).must_equal 'foo=bar'
@@ -308,35 +341,39 @@ describe Rack::Utils do
     Rack::Utils.build_nested_query('foo' => 'bar', 'baz' => '').
       must_equal 'foo=bar&baz='
     Rack::Utils.build_nested_query('foo' => ['1', '2']).
-      must_equal 'foo[]=1&foo[]=2'
+      must_equal 'foo%5B%5D=1&foo%5B%5D=2'
     Rack::Utils.build_nested_query('foo' => 'bar', 'baz' => ['1', '2', '3']).
-      must_equal 'foo=bar&baz[]=1&baz[]=2&baz[]=3'
+      must_equal 'foo=bar&baz%5B%5D=1&baz%5B%5D=2&baz%5B%5D=3'
     Rack::Utils.build_nested_query('foo' => ['bar'], 'baz' => ['1', '2', '3']).
-      must_equal 'foo[]=bar&baz[]=1&baz[]=2&baz[]=3'
+      must_equal 'foo%5B%5D=bar&baz%5B%5D=1&baz%5B%5D=2&baz%5B%5D=3'
     Rack::Utils.build_nested_query('foo' => ['bar'], 'baz' => ['1', '2', '3']).
-      must_equal 'foo[]=bar&baz[]=1&baz[]=2&baz[]=3'
+      must_equal 'foo%5B%5D=bar&baz%5B%5D=1&baz%5B%5D=2&baz%5B%5D=3'
     Rack::Utils.build_nested_query('x' => { 'y' => { 'z' => '1' } }).
-      must_equal 'x[y][z]=1'
+      must_equal 'x%5By%5D%5Bz%5D=1'
     Rack::Utils.build_nested_query('x' => { 'y' => { 'z' => ['1'] } }).
-      must_equal 'x[y][z][]=1'
+      must_equal 'x%5By%5D%5Bz%5D%5B%5D=1'
     Rack::Utils.build_nested_query('x' => { 'y' => { 'z' => ['1', '2'] } }).
-      must_equal 'x[y][z][]=1&x[y][z][]=2'
+      must_equal 'x%5By%5D%5Bz%5D%5B%5D=1&x%5By%5D%5Bz%5D%5B%5D=2'
     Rack::Utils.build_nested_query('x' => { 'y' => [{ 'z' => '1' }] }).
-      must_equal 'x[y][][z]=1'
+      must_equal 'x%5By%5D%5B%5D%5Bz%5D=1'
     Rack::Utils.build_nested_query('x' => { 'y' => [{ 'z' => ['1'] }] }).
-      must_equal 'x[y][][z][]=1'
+      must_equal 'x%5By%5D%5B%5D%5Bz%5D%5B%5D=1'
     Rack::Utils.build_nested_query('x' => { 'y' => [{ 'z' => '1', 'w' => '2' }] }).
-      must_equal 'x[y][][z]=1&x[y][][w]=2'
+      must_equal 'x%5By%5D%5B%5D%5Bz%5D=1&x%5By%5D%5B%5D%5Bw%5D=2'
     Rack::Utils.build_nested_query('x' => { 'y' => [{ 'v' => { 'w' => '1' } }] }).
-      must_equal 'x[y][][v][w]=1'
+      must_equal 'x%5By%5D%5B%5D%5Bv%5D%5Bw%5D=1'
     Rack::Utils.build_nested_query('x' => { 'y' => [{ 'z' => '1', 'v' => { 'w' => '2' } }] }).
-      must_equal 'x[y][][z]=1&x[y][][v][w]=2'
+      must_equal 'x%5By%5D%5B%5D%5Bz%5D=1&x%5By%5D%5B%5D%5Bv%5D%5Bw%5D=2'
     Rack::Utils.build_nested_query('x' => { 'y' => [{ 'z' => '1' }, { 'z' => '2' }] }).
-      must_equal 'x[y][][z]=1&x[y][][z]=2'
+      must_equal 'x%5By%5D%5B%5D%5Bz%5D=1&x%5By%5D%5B%5D%5Bz%5D=2'
     Rack::Utils.build_nested_query('x' => { 'y' => [{ 'z' => '1', 'w' => 'a' }, { 'z' => '2', 'w' => '3' }] }).
-      must_equal 'x[y][][z]=1&x[y][][w]=a&x[y][][z]=2&x[y][][w]=3'
+      must_equal 'x%5By%5D%5B%5D%5Bz%5D=1&x%5By%5D%5B%5D%5Bw%5D=a&x%5By%5D%5B%5D%5Bz%5D=2&x%5By%5D%5B%5D%5Bw%5D=3'
     Rack::Utils.build_nested_query({ "foo" => ["1", ["2"]] }).
-      must_equal 'foo[]=1&foo[][]=2'
+      must_equal 'foo%5B%5D=1&foo%5B%5D%5B%5D=2'
+
+    # A nested hash is the same as string keys with brackets.
+    Rack::Utils.build_nested_query('foo' => { 'bar' => 'baz' }).
+      must_equal Rack::Utils.build_nested_query('foo[bar]' => 'baz')
 
     lambda { Rack::Utils.build_nested_query("foo=bar") }.
       must_raise(ArgumentError).
@@ -344,7 +381,7 @@ describe Rack::Utils do
   end
 
   it 'performs the inverse function of #parse_nested_query' do
-    [{ "foo" => nil, "bar" => "" },
+    [{ "bar" => "" },
       { "foo" => "bar", "baz" => "" },
       { "foo" => ["1", "2"] },
       { "foo" => "bar", "baz" => ["1", "2", "3"] },
@@ -392,6 +429,45 @@ describe Rack::Utils do
     ]
   end
 
+  it "parses RFC 7239 Forwarded header" do
+    Rack::Utils.forwarded_values('for=3.4.5.6').must_equal({
+      for: [ '3.4.5.6' ],
+    })
+
+    Rack::Utils.forwarded_values(';;;for=3.4.5.6,,').must_equal({
+      for: [ '3.4.5.6' ],
+    })
+
+    Rack::Utils.forwarded_values('for=3.4.5.6').must_equal({
+      for: [ '3.4.5.6' ],
+    })
+
+    Rack::Utils.forwarded_values('for =  3.4.5.6').must_equal({
+      for: [ '3.4.5.6' ],
+    })
+
+    Rack::Utils.forwarded_values('for="3.4.5.6"').must_equal({
+      for: [ '3.4.5.6' ],
+    })
+
+    Rack::Utils.forwarded_values('for=3.4.5.6;proto=https').must_equal({
+      for: [ '3.4.5.6' ],
+      proto: [ 'https' ]
+    })
+
+    Rack::Utils.forwarded_values('for=3.4.5.6; proto=http, proto=https').must_equal({
+      for: [ '3.4.5.6' ],
+      proto: [ 'http', 'https' ]
+    })
+
+    Rack::Utils.forwarded_values('for=3.4.5.6; proto=http, proto=https; for=1.2.3.4').must_equal({
+      for: [ '3.4.5.6', '1.2.3.4' ],
+      proto: [ 'http', 'https' ]
+    })
+
+    Rack::Utils.forwarded_values('for=3.4.5.6; foo=bar').must_be_nil
+  end
+
   it "select best quality match" do
     Rack::Utils.best_q_match("text/html", %w[text/html]).must_equal "text/html"
 
@@ -463,6 +539,7 @@ describe Rack::Utils do
   it "should perform constant time string comparison" do
     Rack::Utils.secure_compare('a', 'a').must_equal true
     Rack::Utils.secure_compare('a', 'b').must_equal false
+    Rack::Utils.secure_compare('a', 'bb').must_equal false
   end
 
   it "return status code for integer" do
@@ -487,10 +564,6 @@ describe Rack::Utils do
     Rack::Utils.rfc2822(Time.at(0).gmtime).must_equal "Thu, 01 Jan 1970 00:00:00 -0000"
   end
 
-  it "return rfc2109 format from rfc2109 helper" do
-    Rack::Utils.rfc2109(Time.at(0).gmtime).must_equal "Thu, 01-Jan-1970 00:00:00 GMT"
-  end
-
   it "clean directory traversal" do
     Rack::Utils.clean_path_info("/cgi/../cgi/test").must_equal "/cgi/test"
     Rack::Utils.clean_path_info(".").must_be_empty
@@ -513,6 +586,9 @@ end
 
 describe Rack::Utils, "cookies" do
   it "parses cookies" do
+    env = Rack::MockRequest.env_for("", "HTTP_COOKIE" => "a=b; ; c=d")
+    Rack::Utils.parse_cookies(env).must_equal({ "a" => "b", "c" => "d" })
+
     env = Rack::MockRequest.env_for("", "HTTP_COOKIE" => "zoo=m")
     Rack::Utils.parse_cookies(env).must_equal({ "zoo" => "m" })
 
@@ -536,236 +612,192 @@ describe Rack::Utils, "cookies" do
     cookies.must_equal({ "%66oo" => "baz", "foo" => "bar" })
   end
 
-  it "adds new cookies to nil header" do
+  it "generates appropriate cookie header value" do
+    Rack::Utils.set_cookie_header('name', 'value').must_equal 'name=value'
+    Rack::Utils.set_cookie_header('name', %w[value]).must_equal 'name=value'
+    Rack::Utils.set_cookie_header('name', %w[va ue]).must_equal 'name=va&ue'
+  end
+
+  deprecated "adds new cookies to nil header" do
     Rack::Utils.add_cookie_to_header(nil, 'name', 'value').must_equal 'name=value'
   end
 
-  it "adds new cookies to blank header" do
+  deprecated "adds new cookies to blank header" do
     header = ''
     Rack::Utils.add_cookie_to_header(header, 'name', 'value').must_equal 'name=value'
     header.must_equal ''
   end
 
-  it "adds new cookies to string header" do
+  deprecated "adds new cookies to string header" do
     header = 'existing-cookie'
-    Rack::Utils.add_cookie_to_header(header, 'name', 'value').must_equal "existing-cookie\nname=value"
+    Rack::Utils.add_cookie_to_header(header, 'name', 'value').must_equal ["existing-cookie", "name=value"]
     header.must_equal 'existing-cookie'
   end
 
-  it "adds new cookies to array header" do
+  deprecated "adds new cookies to array header" do
     header = %w[ existing-cookie ]
-    Rack::Utils.add_cookie_to_header(header, 'name', 'value').must_equal "existing-cookie\nname=value"
+    Rack::Utils.add_cookie_to_header(header, 'name', 'value').must_equal ["existing-cookie", "name=value"]
     header.must_equal %w[ existing-cookie ]
   end
 
-  it "adds new cookies to an unrecognized header" do
+  deprecated "adds new cookies to an unrecognized header" do
     lambda {
       Rack::Utils.add_cookie_to_header(Object.new, 'name', 'value')
     }.must_raise ArgumentError
   end
 
   it "sets and deletes cookies in header hash" do
-    header = { 'Set-Cookie' => '' }
-    Rack::Utils.set_cookie_header!(header, 'name', 'value').must_be_nil
-    header['Set-Cookie'].must_equal 'name=value'
-    Rack::Utils.set_cookie_header!(header, 'name2', 'value2').must_be_nil
-    header['Set-Cookie'].must_equal "name=value\nname2=value2"
-    Rack::Utils.set_cookie_header!(header, 'name2', 'value3').must_be_nil
-    header['Set-Cookie'].must_equal "name=value\nname2=value2\nname2=value3"
-
-    Rack::Utils.delete_cookie_header!(header, 'name2').must_be_nil
-    header['Set-Cookie'].must_equal "name=value\nname2=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
-    Rack::Utils.delete_cookie_header!(header, 'name').must_be_nil
-    header['Set-Cookie'].must_equal "name2=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT\nname=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
-
-    header = { 'Set-Cookie' => nil }
-    Rack::Utils.delete_cookie_header!(header, 'name').must_be_nil
-    header['Set-Cookie'].must_equal "name=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
-
-    header = { 'Set-Cookie' => [] }
-    Rack::Utils.delete_cookie_header!(header, 'name').must_be_nil
-    header['Set-Cookie'].must_equal "name=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
+    headers = {}
+    Rack::Utils.set_cookie_header!(headers, 'name', 'value')
+    headers['set-cookie'].must_equal 'name=value'
+    Rack::Utils.set_cookie_header!(headers, 'name2', 'value2')
+    headers['set-cookie'].must_equal ['name=value', 'name2=value2']
+    Rack::Utils.set_cookie_header!(headers, 'name2', 'value3')
+    headers['set-cookie'].must_equal ['name=value', 'name2=value2', 'name2=value3']
   end
 
+  it "encodes cookie key values by default" do
+    Rack::Utils.set_cookie_header('na e', 'value').must_equal 'na+e=value'
+  end
+
+  it "does not encode cookie key values if :escape_key is false" do
+    Rack::Utils.set_cookie_header('na e', value: 'value', escape_key: false).must_equal 'na e=value'
+  end
+
+  it "deletes cookies in header field" do
+    header = []
+
+    Rack::Utils.delete_set_cookie_header!(header, 'name2')
+    header.must_equal [
+      "name2=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
+    ]
+
+    Rack::Utils.delete_set_cookie_header!(header, 'name')
+    header.must_equal [
+      "name2=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT",
+      "name=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
+    ]
+  end
+
+  it "deletes cookies in header field with domain" do
+    header = []
+
+    Rack::Utils.delete_set_cookie_header!(header, 'name', {domain: "mydomain.com"})
+    header.must_equal [
+      "name=; domain=mydomain.com; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
+    ]
+  end
+
+  it "deletes cookies in header field with path" do
+    header = []
+
+    Rack::Utils.delete_set_cookie_header!(header, 'name', {path: "/a/b"})
+    header.must_equal [
+      "name=; path=/a/b; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
+    ]
+  end
+
+  it "sets and deletes cookies in header hash" do
+    header = { 'set-cookie' => nil }
+    Rack::Utils.delete_cookie_header!(header, 'name').must_be_nil
+    header['set-cookie'].must_equal "name=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
+
+    header = { 'set-cookie' => nil }
+    Rack::Utils.delete_cookie_header!(header, 'name').must_be_nil
+    header['set-cookie'].must_equal "name=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
+  end
+
+  deprecated "sets deleted cookie" do
+    Rack::Utils.make_delete_cookie_header(nil, 'name', {}).
+      must_equal "name=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
+    Rack::Utils.add_remove_cookie_to_header(nil, 'name').
+      must_equal "name=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
+  end
 end
 
-describe Rack::Utils, "byte_range" do
+describe Rack::Utils, "get_byte_ranges" do
+  deprecated "pase simple byte ranges from env" do
+    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=123-456" }, 500).must_equal [(123..456)]
+  end
+
   it "ignore missing or syntactically invalid byte ranges" do
-    Rack::Utils.byte_ranges({}, 500).must_be_nil
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "foobar" }, 500).must_be_nil
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "furlongs=123-456" }, 500).must_be_nil
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=" }, 500).must_be_nil
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=-" }, 500).must_be_nil
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=123,456" }, 500).must_be_nil
+    Rack::Utils.get_byte_ranges(nil, 500).must_be_nil
+    Rack::Utils.get_byte_ranges("foobar", 500).must_be_nil
+    Rack::Utils.get_byte_ranges("furlongs=123-456", 500).must_be_nil
+    Rack::Utils.get_byte_ranges("bytes=", 500).must_be_nil
+    Rack::Utils.get_byte_ranges("bytes=-", 500).must_be_nil
+    Rack::Utils.get_byte_ranges("bytes=123,456", 500).must_be_nil
     # A range of non-positive length is syntactically invalid and ignored:
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=456-123" }, 500).must_be_nil
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=456-455" }, 500).must_be_nil
+    Rack::Utils.get_byte_ranges("bytes=456-123", 500).must_be_nil
+    Rack::Utils.get_byte_ranges("bytes=456-455", 500).must_be_nil
   end
 
   it "parse simple byte ranges" do
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=123-456" }, 500).must_equal [(123..456)]
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=123-" }, 500).must_equal [(123..499)]
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=-100" }, 500).must_equal [(400..499)]
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=0-0" }, 500).must_equal [(0..0)]
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=499-499" }, 500).must_equal [(499..499)]
+    Rack::Utils.get_byte_ranges("bytes=123-456", 500).must_equal [(123..456)]
+    Rack::Utils.get_byte_ranges("bytes=123-", 500).must_equal [(123..499)]
+    Rack::Utils.get_byte_ranges("bytes=-100", 500).must_equal [(400..499)]
+    Rack::Utils.get_byte_ranges("bytes=0-0", 500).must_equal [(0..0)]
+    Rack::Utils.get_byte_ranges("bytes=499-499", 500).must_equal [(499..499)]
   end
 
   it "parse several byte ranges" do
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=500-600,601-999" }, 1000).must_equal [(500..600), (601..999)]
+    Rack::Utils.get_byte_ranges("bytes=500-600,601-999", 1000).must_equal [(500..600), (601..999)]
   end
 
   it "truncate byte ranges" do
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=123-999" }, 500).must_equal [(123..499)]
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=600-999" }, 500).must_equal []
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=-999" }, 500).must_equal [(0..499)]
+    Rack::Utils.get_byte_ranges("bytes=123-999", 500).must_equal [(123..499)]
+    Rack::Utils.get_byte_ranges("bytes=600-999", 500).must_equal []
+    Rack::Utils.get_byte_ranges("bytes=-999", 500).must_equal [(0..499)]
   end
 
   it "ignore unsatisfiable byte ranges" do
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=500-501" }, 500).must_equal []
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=500-" }, 500).must_equal []
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=999-" }, 500).must_equal []
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=-0" }, 500).must_equal []
+    Rack::Utils.get_byte_ranges("bytes=500-501", 500).must_equal []
+    Rack::Utils.get_byte_ranges("bytes=500-", 500).must_equal []
+    Rack::Utils.get_byte_ranges("bytes=999-", 500).must_equal []
+    Rack::Utils.get_byte_ranges("bytes=-0", 500).must_equal []
   end
 
   it "handle byte ranges of empty files" do
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=123-456" }, 0).must_equal []
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=0-" }, 0).must_equal []
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=-100" }, 0).must_equal []
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=0-0" }, 0).must_equal []
-    Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=-0" }, 0).must_equal []
+    Rack::Utils.get_byte_ranges("bytes=123-456", 0).must_equal []
+    Rack::Utils.get_byte_ranges("bytes=0-", 0).must_equal []
+    Rack::Utils.get_byte_ranges("bytes=-100", 0).must_equal []
+    Rack::Utils.get_byte_ranges("bytes=0-0", 0).must_equal []
+    Rack::Utils.get_byte_ranges("bytes=-0", 0).must_equal []
   end
 end
 
 describe Rack::Utils::HeaderHash do
-  it "retain header case" do
+  deprecated ".[] returns Rack::Headers as is if not frozen" do
+    h1 = Rack::Headers["Content-MD5" => "d5ff4e2a0 ..."]
+    h2 = Rack::Utils::HeaderHash[h1]
+    h2.must_be_same_as h1
+    h3 = Rack::Utils::HeaderHash[h1.freeze]
+    h3.wont_be_same_as h1
+    h3.must_equal h1
+  end
+
+  deprecated ".[] returns instance of Rack::Headers" do
+    h = Rack::Utils::HeaderHash["Content-MD5" => "d5ff4e2a0 ..."]
+    h.must_be_kind_of Rack::Headers
+    h['content-md5'].must_equal "d5ff4e2a0 ..."
+
+    h = Rack::Utils::HeaderHash[[["Content-MD5","d5ff4e2a0 ..."]]]
+    h.must_be_kind_of Rack::Headers
+    h['content-md5'].must_equal "d5ff4e2a0 ..."
+  end
+
+  deprecated ".new returns instance of Rack::Headers" do
     h = Rack::Utils::HeaderHash.new("Content-MD5" => "d5ff4e2a0 ...")
-    h['ETag'] = 'Boo!'
-    h.to_hash.must_equal "Content-MD5" => "d5ff4e2a0 ...", "ETag" => 'Boo!'
+    h.must_be_kind_of Rack::Headers
+    h['content-md5'].must_equal "d5ff4e2a0 ..."
+
+    h = Rack::Utils::HeaderHash.new([["Content-MD5","d5ff4e2a0 ..."]])
+    h.must_be_kind_of Rack::Headers
+    h['content-md5'].must_equal "d5ff4e2a0 ..."
   end
 
-  it "check existence of keys case insensitively" do
-    h = Rack::Utils::HeaderHash.new("Content-MD5" => "d5ff4e2a0 ...")
-    h.must_include 'content-md5'
-    h.wont_include 'ETag'
-  end
-
-  it "create deep HeaderHash copy on dup" do
-    h1 = Rack::Utils::HeaderHash.new("Content-MD5" => "d5ff4e2a0 ...")
-    h2 = h1.dup
-
-    h1.must_include 'content-md5'
-    h2.must_include 'content-md5'
-
-    h2.delete("Content-MD5")
-
-    h2.wont_include 'content-md5'
-    h1.must_include 'content-md5'
-  end
-
-  it "merge case-insensitively" do
-    h = Rack::Utils::HeaderHash.new("ETag" => 'HELLO', "content-length" => '123')
-    merged = h.merge("Etag" => 'WORLD', 'Content-Length' => '321', "Foo" => 'BAR')
-    merged.must_equal "Etag" => 'WORLD', "Content-Length" => '321', "Foo" => 'BAR'
-  end
-
-  it "overwrite case insensitively and assume the new key's case" do
-    h = Rack::Utils::HeaderHash.new("Foo-Bar" => "baz")
-    h["foo-bar"] = "bizzle"
-    h["FOO-BAR"].must_equal "bizzle"
-    h.length.must_equal 1
-    h.to_hash.must_equal "foo-bar" => "bizzle"
-  end
-
-  it "be converted to real Hash" do
-    h = Rack::Utils::HeaderHash.new("foo" => "bar")
-    h.to_hash.must_be_instance_of Hash
-  end
-
-  it "convert Array values to Strings when converting to Hash" do
-    h = Rack::Utils::HeaderHash.new("foo" => ["bar", "baz"])
-    h.to_hash.must_equal({ "foo" => "bar\nbaz" })
-  end
-
-  it "replace hashes correctly" do
-    h = Rack::Utils::HeaderHash.new("Foo-Bar" => "baz")
-    j = { "foo" => "bar" }
-    h.replace(j)
-    h["foo"].must_equal "bar"
-  end
-
-  it "be able to delete the given key case-sensitively" do
-    h = Rack::Utils::HeaderHash.new("foo" => "bar")
-    h.delete("foo")
-    h["foo"].must_be_nil
-    h["FOO"].must_be_nil
-  end
-
-  it "be able to delete the given key case-insensitively" do
-    h = Rack::Utils::HeaderHash.new("foo" => "bar")
-    h.delete("FOO")
-    h["foo"].must_be_nil
-    h["FOO"].must_be_nil
-  end
-
-  it "return the deleted value when #delete is called on an existing key" do
-    h = Rack::Utils::HeaderHash.new("foo" => "bar")
-    h.delete("Foo").must_equal "bar"
-  end
-
-  it "return nil when #delete is called on a non-existent key" do
-    h = Rack::Utils::HeaderHash.new("foo" => "bar")
-    h.delete("Hello").must_be_nil
-  end
-
-  it "dups given HeaderHash" do
-    a = Rack::Utils::HeaderHash.new("foo" => "bar")
-    b = Rack::Utils::HeaderHash.new(a)
-    b.object_id.wont_equal a.object_id
-    b.must_equal a
-  end
-
-  it "convert Array values to Strings when responding to #each" do
-    h = Rack::Utils::HeaderHash.new("foo" => ["bar", "baz"])
-    h.each do |k, v|
-      k.must_equal "foo"
-      v.must_equal "bar\nbaz"
-    end
-  end
-
-  it "not create headers out of thin air" do
-    h = Rack::Utils::HeaderHash.new
-    h['foo']
-    h['foo'].must_be_nil
-    h.wont_include 'foo'
-  end
-
-  it "uses memoized header hash" do
-    env = {}
-    headers = Rack::Utils::HeaderHash.new({ 'content-type' => "text/plain", "content-length" => "3" })
-
-    app = lambda do |env|
-      [200, headers, []]
-    end
-
-    app = Rack::ContentLength.new(app)
-
-    response = app.call(env)
-    assert_same response[1], headers
-  end
-
-  it "duplicates header hash" do
-    env = {}
-    headers = Rack::Utils::HeaderHash.new({ 'content-type' => "text/plain", "content-length" => "3" })
-    headers.freeze
-
-    app = lambda do |env|
-      [200, headers, []]
-    end
-
-    app = Rack::ContentLength.new(app)
-
-    response = app.call(env)
-    refute_same response[1], headers
+  it ".allocate raises" do
+    proc { Rack::Utils::HeaderHash.allocate }.must_raise TypeError
   end
 end
 
@@ -779,7 +811,7 @@ describe Rack::Utils::Context do
   test_target1 = proc{|e| e.to_s + ' world' }
   test_target2 = proc{|e| e.to_i + 2 }
   test_target3 = proc{|e| nil }
-  test_target4 = proc{|e| [200, { 'Content-Type' => 'text/plain', 'Content-Length' => '0' }, ['']] }
+  test_target4 = proc{|e| [200, { 'content-type' => 'text/plain', 'content-length' => '0' }, ['']] }
   test_app = ContextTest.new test_target4
 
   it "set context correctly" do
@@ -823,4 +855,10 @@ describe Rack::Utils::Context do
     r5.status.must_equal 200
     r4.body.must_equal r5.body
   end
+
+  it "raises for invalid context" do
+    proc do
+      Rack::Utils::Context.new(nil, test_target1)
+    end.must_raise RuntimeError
+  end
 end
